#!/bin/bash

LIST_OF_ERROR_CODES_TO_WORK_WITH='403 404 400 500'

GREP_BIN=`which grep`
GREP_BASIC_CMD="$GREP_BIN -i"
GREP_IGNORE_COMMENTS="$GREP_BASIC_CMD -v \#"
GREP_find_AS_A_COMMENT="$GREP_BASIC_CMD \#"
GREP_find_LINE_NUMBER="$GREP_BASIC_CMD -n"

sed_BIN=`which sed`
sed_BASIC_CMD="$sed_BIN --follow-symlinks -i"

HARDENED_HTACCESS_FILES_LOCATION=/root/scripts/htaccess

DIRECTIVE_TO_WORK_WITH='ErrorDocument'
CUSTOM_ERROR_FILE_SPEC='/error.htm'
DRUPAL_ERROR_FILE_SPEC='/index.php'

APPEND_AFTER_LINE=16

showCurrentStatus()
{
	# params:
	# $1 - htaccess file spec

	echo -e "\nCurrent Status:"
	echo -e "----------------\n"

	$GREP_BASIC_CMD $DIRECTIVE_TO_WORK_WITH $1

	echo -e "\n"
}

removeAllDirectives()
{
	# params:
	# $1 - htaccess file spec

	$sed_BASIC_CMD "/${DIRECTIVE_TO_WORK_WITH}/d" $1
}

resetDirectivesToDrupalDefault()
{
	# params:
	# $1 - htaccess file spec

	removeAllDirectives $1

	$sed_BASIC_CMD "${APPEND_AFTER_LINE}i $DIRECTIVE_TO_WORK_WITH 404 $DRUPAL_ERROR_FILE_SPEC" $1
}

hardenDirectives()
{
	# params:
	# $1 - htaccess file spec

	removeAllDirectives $1

	SITE_PATH=`dirname $1`

	if ! [ -s "$SITE_PATH""$CUSTOM_ERROR_FILE_SPEC" ]; then
		
		cp $HARDENED_HTACCESS_FILES_LOCATION$CUSTOM_ERROR_FILE_SPEC $SITE_PATH
	fi

	for i in $(echo $LIST_OF_ERROR_CODES_TO_WORK_WITH)
	do
		$sed_BASIC_CMD "${APPEND_AFTER_LINE}i $DIRECTIVE_TO_WORK_WITH $i $CUSTOM_ERROR_FILE_SPEC" $1
	
	done
}

if [ "$1" == "" ]; then
	echo "ERROR! Wrong input."
	echo "Usage: $0 <htaccess_file_to_work_on> [optional: an_action]"
	echo "Example: $0 /data/foi.gov.il/.htaccess add"	

	echo "an_action:"
	echo "add - harden settings using custom errors"
	echo "del - delete all custom errors related settings"
	echo "drupal - reset settings to drupal's default"
	echo "any other word or nothing at all - show current settings status"
	

	EXIT_CODE=1
else
	# check if the given argument is a relative path to a valid file
	if ! [ -s "$1" ]; then
		if [ -s `pwd`/"$1" ]; then
			FILE_SPEC=`pwd`/$1
		else
			FILE_SPEC=""
		fi
	else
		if ! [ -d "$1" ]; then
			FILE_SPEC=$1
		else
			echo "ERROR! the input '$1' is a folder."
			echo "Aborting ..."
		
			exit 1
		fi
	fi

	if ! [ "$FILE_SPEC" == "" ]; then
		
		case $2 in
			"drupal") resetDirectivesToDrupalDefault $FILE_SPEC 
			
			showCurrentStatus $FILE_SPEC 

			;;

			"add") hardenDirectives $FILE_SPEC 
			
			showCurrentStatus $FILE_SPEC 

			;;

			"del") removeAllDirectives $FILE_SPEC
 			
			showCurrentStatus $FILE_SPEC 

			;;

			*) showCurrentStatus $FILE_SPEC 

			;;
		esac

		EXIT_CODE=0
	else
		echo "ERROR! the input '$1' is an invalid non-empty file."
		echo "Aborting ..."
		
		EXIT_CODE=1
	fi
fi

exit $EXIT_CODE
